package ru.dbotthepony.kstarbound

import org.apache.logging.log4j.LogManager
import org.lwjgl.Version
import org.lwjgl.glfw.GLFW.glfwSetWindowShouldClose
import ru.dbotthepony.kstarbound.client.StarboundClient
import ru.dbotthepony.kstarbound.defs.TileDefinition
import ru.dbotthepony.kstarbound.lua.LuaJNI
import ru.dbotthepony.kstarbound.lua.LuaState
import ru.dbotthepony.kstarbound.math.Vector2d
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE_FF
import ru.dbotthepony.kstarbound.world.Chunk
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.entities.Move
import ru.dbotthepony.kstarbound.world.entities.PlayerEntity
import java.io.File

private val LOGGER = LogManager.getLogger()

fun main() {
	LOGGER.info("Running LWJGL ${Version.getVersion()}")

	val client = StarboundClient()

	Starbound.addFilePath(File("./unpacked_assets/"))

	Starbound.initializeGame { finished, replaceStatus, status ->
		client.putDebugLog(status, replaceStatus)
	}

	client.onTermination {
		Starbound.terminateLoading = true
	}

	var chunkA: Chunk<*, *>? = null

	val ent = PlayerEntity(client.world!!)

	Starbound.onInitialize {
		chunkA = client.world!!.computeIfAbsent(ChunkPos(0, 0)).chunk
		val chunkB = client.world!!.computeIfAbsent(ChunkPos(-1, 0)).chunk
		val chunkC = client.world!!.computeIfAbsent(ChunkPos(-2, 0)).chunk

		val tile = Starbound.getTileDefinition("alienrock")

		for (x in -48 .. 48) {
			for (y in 0 .. 20) {
				val chnk = client.world!!.computeIfAbsent(ChunkPos(x, y))

				if (y == 0) {
					for (bx in 0 .. 31) {
						for (by in 0 .. 3) {
							chnk.chunk.foreground[bx, by] = tile
						}
					}
				}
			}
		}

		run {
			val chunk = client.world!!.computeIfAbsent(ChunkPos(-2, 0)).chunk

			for (y in 0 .. CHUNK_SIZE_FF) {
				for (x in 0 .. (CHUNK_SIZE_FF - y)) {
					chunk.foreground[x, y] = tile
				}
			}
		}

		run {
			val chunk = client.world!!.computeIfAbsent(ChunkPos(-3, 0)).chunk

			for (y in 0 .. CHUNK_SIZE_FF) {
				for (x in 0 .. (CHUNK_SIZE_FF - y * 2)) {
					chunk.foreground[x, y] = tile
				}
			}
		}

		for (x in 0 .. 31) {
			for (y in 0 .. 3) {
				chunkA!!.foreground[x, y] = tile
			}
		}

		for (x in 0 .. 31) {
			for (y in 8 .. 9) {
				chunkA!!.foreground[x, y] = tile
			}
		}

		for (x in 0 .. 31) {
			for (y in 0 .. 0) {
				chunkB.foreground[x, y] = tile
			}
		}

		for (x in 4 .. 8) {
			for (y in 4 .. 8) {
				chunkA!!.foreground[x, y] = null as TileDefinition?
			}
		}

		/*val rand = Random()

		for (i in 0 .. 400) {
			chunkA!!.foreground[rand.nextInt(0, CHUNK_SIZE_FF), rand.nextInt(0, CHUNK_SIZE_FF)] = tile
		}*/

		ent.dropToFloor()
	}

	//val rand = Random()

	ent.pos += Vector2d(y = 36.0, x = -10.0)

	client.onDrawGUI {
		client.gl.font.render("${ent.pos}", y = 100f, scale = 0.25f)
		client.gl.font.render("${ent.velocity}", y = 120f, scale = 0.25f)
	}

	client.onPreDrawWorld {
		client.camera.pos.x = ent.pos.x.toFloat()
		client.camera.pos.y = ent.pos.y.toFloat()
	}

	ent.spawn()

	while (client.renderFrame()) {
		Starbound.pollCallbacks()

		//ent.think(client.frameRenderTime)
		client.camera.pos.x = ent.pos.x.toFloat()
		client.camera.pos.y = ent.pos.y.toFloat()

		//println(client.camera.velocity.toDoubleVector() * client.frameRenderTime * 0.1)

		//if (ent.onGround)
			//ent.velocity += client.camera.velocity.toDoubleVector() * client.frameRenderTime * 0.1

		if (client.input.KEY_LEFT_DOWN) {
			ent.moveDirection = Move.MOVE_LEFT
		} else if (client.input.KEY_RIGHT_DOWN) {
			ent.moveDirection = Move.MOVE_RIGHT
		} else {
			ent.moveDirection = Move.STAND_STILL
		}

		if (client.input.KEY_SPACE_PRESSED && ent.onGround) {
			ent.requestJump()
		} else if (client.input.KEY_SPACE_RELEASED) {
			ent.recallJump()
		}

		if (client.input.KEY_ESCAPE_PRESSED) {
			glfwSetWindowShouldClose(client.window, true)
		}

		ent.wantsToDuck = client.input.KEY_DOWN_DOWN

		//if (chunkA != null && glfwGetTime() < 10.0) {
		//	val tile = Starbound.getTileDefinition("alienrock")
			//chunkA!!.foreground[rand.nextInt(0, CHUNK_SIZE_FF), rand.nextInt(0, CHUNK_SIZE_FF)] = tile
		//}
	}
}
